@@ -7,14 +7,14 @@ from mkr.utils.PackOS import BiasPath
 
 
 class PyFile:
-    def __init__(self, root: str, file_path: str):
+    def __init__(self, root: str, file_path: str, encoding='utf-8'):
         # 安全性检查
         bp = BiasPath(root, "")
         dp = BiasPath(file_path, "")
-        assert bp.isdir(), "target path must be a dir, not : " + str(bp)
-        assert bp.isabs(), "target path must be a abspath, not : " + str(bp)
-        assert dp.isfile(), "target path must be a file, not : " + str(dp)
-        assert dp.isabs(), "target path must be a abspath, not : " + str(dp)
+        assert bp.isdir(),  "target path must be a dir, not : "     + str(bp)
+        assert bp.isabs(),  "target path must be a abspath, not : " + str(bp)
+        assert dp.isfile(), "target path must be a file, not : "    + str(dp)
+        assert dp.isabs(),  "target path must be a abspath, not : " + str(dp)
         assert str(dp)[-3:].lower() == '.py', "target path must be a pyfile, not : " + str(dp)
 
         # 初始化
@@ -22,6 +22,7 @@ class PyFile:
         self.rel = []
         self.fname = os.path.basename(file_path)[:-3]
         self._raw_fpath = file_path
+        self.encoding = encoding
 
         path = os.path.dirname(file_path)
         if root != path:
@@ -36,10 +37,17 @@ class PyFile:
                 path = _dir
 
     def read(self):
-        return open(self._raw_fpath, 'r').read()
+        try:
+            return open(self._raw_fpath, 'r', encoding='UTF-8').read()
+        except UnicodeDecodeError as err:
+            raise Exception(f"Unicode Error: <file:{self._raw_fpath}>\n{err}")
 
     def write(self, txt: str):
-        return open(self._raw_fpath, 'w').write(txt)
+        try:
+            return open(self._raw_fpath, 'w', encoding=self.encoding).write(txt)
+        except UnicodeDecodeError as err:
+            raise Exception(f"Unicode Error: <file:{self._raw_fpath}>\n{err}")
+
 
     def __str__(self):
         rels = "-"
@@ -55,7 +63,27 @@ class ImportSegment:
     ORDz = ord('z')
     ORDA = ord('A')
     ORDZ = ord('Z')
-    IMPPATTERN = '[^\n\s:]*f?r?o?m?[\s\w]*import.+\n'
+    FROMPATTERN = '[^\n\s:]*from[\s\w.]*import.+\n'  # --*--
+    IMPPATTERN  = '[^\n\s:]*import.+\n'             # --*--
+    FKEYNAME = "$<restruct>:"
+    FKEYPATTERN = rf'\{FKEYNAME}\d+'                  # --*--
+
+    # --*--
+    _from_replaces, _from_id = {}, 0
+
+    @staticmethod
+    def release():
+        ImportSegment._from_replaces, ImportSegment._from_id = {}, 0
+
+    @property
+    def _id(self):
+        ImportSegment._from_id += 1
+        return ImportSegment._from_id - 1
+
+    @property
+    def _frep(self):
+        return ImportSegment._from_replaces
+    # --*-- /
 
     def __init__(self, segment: str, preix: str):
         self.seg = segment
@@ -63,10 +91,62 @@ class ImportSegment:
 
     @staticmethod
     def replace(match, preix):
-        seg = match.group()[:-1]  # 去掉\n
-        # print('seg: ', seg)
+        """
+        负责对:
+            from ... import xxx
+            import xxx
+        进行响应的函数
+        返回对应的替换值
+        :param match:
+        :param preix:
+        :return:
+        """
+        if isinstance(match, str):
+            seg = match
+        else:
+            seg = match.group()[:-1]  # 去掉\n
+
+        ret = ""
+        if '\n' in seg:
+            for s in seg.split('\n'):
+                if s:
+                    ret += '\n' + ImportSegment.replace(s, preix)
+            return ret
+
         imp_seg = ImportSegment(seg, preix)
-        return imp_seg.solution()
+
+        if imp_seg.isfrom:
+            _id = str(imp_seg._id)
+            key, value = imp_seg.FKEYNAME + _id, imp_seg.solution()
+            imp_seg._frep[_id] = value
+            ret = key + '\n'
+        else:
+            ret = imp_seg.solution()
+
+        print('>>seg: ', seg, "; -- isfrom: ", imp_seg.isfrom, ":: answer is: ", ret)
+        return ret
+
+    @staticmethod
+    def secReplace(match):
+        """
+        负责对:
+            $<restruct>:xxx
+        进行响应的函数
+        返回对应的替换值
+        :param match:
+        :return:
+        """
+        # print("dict:", ImportSegment._from_replaces)
+        seg   = match.group()
+        index = seg.find(':')
+        _id   = seg[index + 1:]
+        if _id.isdigit():
+            value = ImportSegment._from_replaces.get(_id)
+            if value:
+                return value
+        print(f"failed to import with {seg}")
+        return f"raise ImportError('Failed to import from raw key: {seg}')"
+
 
     @property
     def prestr(self):
@@ -147,6 +227,9 @@ class ImportSegment:
             left, right = self.seg[:self.importpos + 6], ''
             for piece in self.seg[self.importpos + 6:].split(','):
                 if not piece: continue
+                if piece.find('*') != -1:
+                    right += f"{piece}, "
+                    continue
                 match  = re.search('\sas\s', piece)
                 index  = match.span()[0] if match else len(piece) + 1
                 module = re.sub('\s', '', piece[:index])
@@ -201,15 +284,24 @@ class ResourceSegment:
         ret = f"{self.seg[:index]}{self.pre}{self.seg[index:]}"
         return ret
 
+def resub(pat, rep_fn, txt):
+    while 1:
+        new = re.sub(pat, rep_fn, txt, 1)
+        if new == txt:
+            return new
+        else:
+            txt = new
 
-def PyRestruct(mod_path: str, new_path: str, run_path:str=None, debug=False):
+def PyRestruct(mod_path: str, new_path: str, run_path:str=None, encoding:str='utf-8', debug=False):
     """
     python代码(关于import路径和资源路径)的重构
     将目标mod移动到当前目录下的
     :param mod_path: abs path. 目标mod根目录
     :param new_path: abs path. 目标mod的新目录
     :param run_path: abs path. 执行处的目录。该目录必须是new_path的一部分
-            _: None  will use os.getcwd()
+                _: None  will use os.getcwd()
+    :param encoding: 解析文件时使用的编码
+                _: utf-8d
     NOTE: these three path must under same disk segment.
     :return:
     """
@@ -226,7 +318,6 @@ def PyRestruct(mod_path: str, new_path: str, run_path:str=None, debug=False):
     assert os.path.isabs(new_path)  , "new mod_path must be a abspath. not "        + new_path
     assert os.path.isdir(new_path)  , "new mod_path must be a dirpath. not "        + new_path
 
-
     # 预处理
     rel = os.path.relpath(new_path, run_path)  # 新目录相对运行目录的路径
     imp_pre = ''  # 一个以.作为分隔符的pypath
@@ -241,15 +332,20 @@ def PyRestruct(mod_path: str, new_path: str, run_path:str=None, debug=False):
         for file in files:
             if file[-3:].lower() == '.py':
                 path = os.path.join(root, file)
-                pyfiles += [PyFile(new_path, path)]
+                pyfiles += [PyFile(new_path, path, encoding)]
 
 
     # restruct body
     sys.path.remove(os.getcwd())
+    ImportSegment.release()
     for py in pyfiles:
         pycode = py.read()
+        pycode = pycode.replace('\n', '\n\n')  # 稀疏化
+        pycode = re.sub(ImportSegment.FROMPATTERN, lambda match: ImportSegment.replace(match, imp_pre), pycode)     # from import segment
         pycode = re.sub(ImportSegment.IMPPATTERN, lambda match: ImportSegment.replace(match, imp_pre), pycode)      # import segment
+        pycode = re.sub(ImportSegment.FKEYPATTERN, ImportSegment.secReplace, pycode)                                # $<restruct> segment
         pycode = re.sub(ResourceSegment.RESPATTERN, lambda match: ResourceSegment.replace(match, res_pre), pycode)  # resource segment
+        pycode = pycode.replace('\n\n', '\n')  # 反稀疏化
         if debug:
             print(pycode)
         else:
